package de.unisaar.jfap.ex6.calculator;

import java.io.IOException;
import java.io.StringReader;
import java.text.ParseException;

/**
 * Provide method to parse and evaluate an arithmetic expression.
 *
 * @author Bernd Kiefer, DFKI
 * @author Jörg Steffen, DFKI
 */
public abstract class Calculator {

  private Calculator() {
    // private constructor to enforce noninstantiability
  }

  /**
   * Read an arithmetic expression and evaluate it.
   *
   * @param expression the expression to evaluate
   * @throws ParseException if expression is not well formed
   * @throws IOException if reading fails
   */
  public static double parseAndEvaluate(String expression) throws ParseException, IOException {
    Parser parser = new Parser(new StringReader(expression));
    if (parser.parse()) {
      return parser.getExpression().evaluate();
    }
    throw new ParseException("Can not be parsed", 0);
  }
}
