package de.unisaar.jfap.ex6.calculator;

import de.unisaar.jfap.ex6.operator.Add;
import de.unisaar.jfap.ex6.operator.Constant;
import de.unisaar.jfap.ex6.operator.Divide;
import de.unisaar.jfap.ex6.operator.Multiply;
import de.unisaar.jfap.ex6.operator.Operator;
import de.unisaar.jfap.ex6.operator.Subtract;
import de.unisaar.jfap.ex6.operator.UnaryMinus;

/**
 * Factory to generate {@link Operator} instances.
 *
 * @author Bernd Kiefer, DFKI
 * @author Jörg Steffen, DFKI
 */
public abstract class OperatorFactory {

  private OperatorFactory() {
    // private constructor to enforce noninstantiability
  }

  /**
   * Create operator based on the given operator using the given arguments.
   *
   * @param op the operator or constant
   * @param args a variable number of arguments
   * @return a concrete implementation of the {@link Operator} interface
   */
  public static Operator getOperator(String op, Operator... args) {

    switch (args.length) {
      case 0:
        // operator contains the constant
        return new Constant(op);
      case 1:
        if (op.equals("-")) {
          return new UnaryMinus(args[0]);
        } else {
          throw new RuntimeException(op + " not defined for one argument");
        }
      case 2:
        switch (op) {
          case "+":
            return new Add(args[0], args[1]);
          case "-":
            return new Subtract(args[0], args[1]);
          case "*":
            return new Multiply(args[0], args[1]);
          case "/":
            return new Divide(args[0], args[1]);
          default:
            throw new RuntimeException(op + " not defined for two arguments");
        }
      default:
        throw new RuntimeException(args.length + " arguments not allowed");
    }
  }
}
