package de.unisaar.jfap.ex6.monitor;

import java.util.ArrayList;
import java.util.List;

/**
 * A car that can be monitored for price changes.
 *
 * @author Jörg Steffen, DFKI
 */
public class Car implements Product {

  private List<Observer> observers = new ArrayList<>();
  private int price;

  /**
   * Create a new car with the given initial price.
   *
   * @param price the initial price
   */
  public Car(int price) {
    this.price = price;
  }

  @Override
  public void register(Observer observer) {
    this.observers.add(observer);
  }

  @Override
  public void unregister(Observer observer) {
    this.observers.remove(observer);
  }

  @Override
  public void notifyObservers() {
    for (Observer oneObserver : this.observers) {
      oneObserver.update(this);
    }
  }

  public void setPrice(int price) {
    this.price = price;
    notifyObservers();
  }

  @Override
  public int getPrice() {
    return this.price;
  }
}
