package de.unisaar.jfap.ex6.monitor;

/**
 * Price monitor for {@link Product}s.
 *
 * @author Jörg Steffen, DFKI
 */
public class PriceMonitor implements Observer {

  // if product price is below this threshold, we consider it a bargain
  private int threshold;
  // flag indicating bargain
  private boolean bargain;

  /**
   * Create a new monitor for the given threshold.
   *
   * @param threshold the threshold
   */
  public PriceMonitor(int threshold) {
    this.threshold = threshold;
    this.bargain = false;
  }

  @Override
  public void update(Product product) {
    if (product.getPrice() < this.threshold) {
      System.out.println(String.format("Alert: price is below threshold %,d", this.threshold));
      this.bargain = true;
    } else {
      System.out.println(String.format("price is above threshold %,d", this.threshold));
      this.bargain = false;
    }
  }

  /** @return a flag indicating if the monitored product is a bargain */
  public boolean isBargain() {
    return this.bargain;
  }
}
