package de.unisaar.jfap.ex6.operator;

/**
 * Abstract implementation of the {@link Operator} interface for binary operators. A binary operator
 * is applied to a left and right argument that are operators themselves.
 *
 * @author Bernd Kiefer, DFKI
 * @author Jörg Steffen, DFKI
 */
public abstract class BinaryOperator implements Operator {

  protected Operator left;
  protected Operator right;

  /**
   * Create a binary operator with the given arguments.
   *
   * @param left the left argument
   * @param right the right argument
   */
  protected BinaryOperator(Operator left, Operator right) {
    this.left = left;
    this.right = right;
  }

  /**
   * Convenience method to be used in the toString method of subclasses.
   *
   * @param opChar character with the operator.
   * @return string representation
   */
  protected String toString(char opChar) {
    return String.format("(%s %s %s)", this.left, opChar, this.right);
  }
}
