package de.unisaar.jfap.ex6.calculator;

import static org.assertj.core.api.Assertions.assertThat;
import static org.assertj.core.api.Assertions.assertThatExceptionOfType;

import de.unisaar.jfap.ex6.operator.Add;
import de.unisaar.jfap.ex6.operator.Constant;
import de.unisaar.jfap.ex6.operator.Divide;
import de.unisaar.jfap.ex6.operator.Multiply;
import de.unisaar.jfap.ex6.operator.Operator;
import de.unisaar.jfap.ex6.operator.Subtract;
import de.unisaar.jfap.ex6.operator.UnaryMinus;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.Test;

class OperatorFactoryTest {

  static Constant c3;
  static Constant c4;

  @BeforeAll
  static void init() {
    c3 = new Constant("3");
    c4 = new Constant("4");
  }

  @Test
  void testConstant() {
    Operator op = OperatorFactory.getOperator("3");
    assertThat(op).isInstanceOf(Constant.class);
    assertThat(op.evaluate()).isEqualTo(3.0);
  }

  @Test
  void testUnaryMinus() {
    Operator op = OperatorFactory.getOperator("-", c3);
    assertThat(op).isInstanceOf(UnaryMinus.class);
    assertThat(op.evaluate()).isEqualTo(-3.0);
  }

  @Test
  void testAdd() {
    Operator op = OperatorFactory.getOperator("+", c3, c4);
    assertThat(op).isInstanceOf(Add.class);
    assertThat(op.evaluate()).isEqualTo(7.0);
  }

  @Test
  void testSubtract() {
    Operator op = OperatorFactory.getOperator("-", c3, c4);
    assertThat(op).isInstanceOf(Subtract.class);
    assertThat(op.evaluate()).isEqualTo(-1.0);
  }

  @Test
  void testMultiply() {
    Operator op = OperatorFactory.getOperator("*", c3, c4);
    assertThat(op).isInstanceOf(Multiply.class);
    assertThat(op.evaluate()).isEqualTo(12.0);
  }

  @Test
  void testDivide() {
    Operator op = OperatorFactory.getOperator("/", c3, c4);
    assertThat(op).isInstanceOf(Divide.class);
    assertThat(op.evaluate()).isEqualTo(0.75);
  }

  @Test
  void testErrors() {
    assertThatExceptionOfType(RuntimeException.class)
        .isThrownBy(() -> OperatorFactory.getOperator("+", c3))
        .withMessage("+ not defined for one argument");
    assertThatExceptionOfType(RuntimeException.class)
        .isThrownBy(() -> OperatorFactory.getOperator("^", c3, c4))
        .withMessage("^ not defined for two arguments");
    assertThatExceptionOfType(RuntimeException.class)
        .isThrownBy(() -> OperatorFactory.getOperator("+", c3, c4, c3))
        .withMessage("3 arguments not allowed");
  }
}
